@@ -5,11 +5,12 @@ use Tk;
 use Tk::Text;
 use Tk::FileSelect;
 use Pod::Text;
+use File::Basename;
 
 # Global parameters to be changed if needed
 
-# The variables we want to be safed in the configuration file
-@SaveVariables = qw(delta begin_from_bottom viewer
+# The variables we want to be saved in the configuration file
+@SaveVariables = qw(delta begin_from_bottom viewer converter
 		    radius_inner tickmarks 
 		    print_circle_boundaries
 		    begin_from_left[0] begin_from_left[1]
@@ -38,6 +39,10 @@ use Pod::Text;
 
 use vars (@SaveVariables);
 
+@FormVariables = qw(form_name delta_form begin_from_bottom_form);
+
+use vars (@FormVariables);
+
 # Then we get some default values
 $pi = 3.14159;
 
@@ -47,7 +52,7 @@ default_text();
 defaults();
 default_colors();
 
-# read the command-line for a additional configuration file
+# read the command-line for an additional configuration file
 
 my ($commandline) = @ARGV;
 
@@ -61,7 +66,7 @@ else {
 # then we need the main-window
 $mw = MainWindow -> new;
 $mw -> title("cd-circleprint");
-# Something which some windowmanager semems to nedd
+# Something which some windowmanager semems to need
 $mw -> geometry("+0+0");
 
 # some key-bindings
@@ -86,6 +91,12 @@ $menu_frame -> Menubutton(-text => "File",
 					 ['command' => "save configuration",
 					  -command => \&write_configuration,
 					  -underline => 0],
+					 ['command' => "load form data",
+					  -command => \&read_form_data,
+					  -underline => 6],
+					 ['command' => "save form data",
+					  -command => \&write_form_data,
+					  -underline => 7],
 					 "-",
 					 ['command' => "save label",
 					 -command => \&save_ps_file,
@@ -95,7 +106,7 @@ $menu_frame -> Menubutton(-text => "File",
 					  -command => \&preview,
 					  -underline => 0],
 					 "-",
-					 ['command' => "exit C-q",
+					 ['command' => "exit\ C-q",
 					  -command => sub{exit},
 					  -underline => 1]
 				       ]) -> pack(-side => 'left');
@@ -140,6 +151,10 @@ $menu_options -> command(-label => "Previewer",
 			 -command => \&get_previewer,
 			 -underline => 0);
 
+$menu_options -> command(-label => "Graphics converter",
+			 -command => \&get_converter,
+			 -underline => 0);
+
 $menu_options -> command(-label => "Load options",
 			 -command => \&read_configuration,
 			 -underline => 0);
@@ -151,6 +166,9 @@ $menu_options -> command(-label => "Save options",
 $menu_frame -> Button(-text => "label", -command => \&print_label) -> pack(-side => 'left');
 $menu_frame -> Button(-text => "preview", -command => \&preview) -> pack(-side => 'left');
 
+# Test for Canvas don't use it
+#$menu_frame -> Button(-text => "CANVAS", -command => \&create_canvas) -> pack(-side => 'left');
+
 ## The help entry
 $menu_help = $menu_frame -> Menubutton(-text => "Help",
 				       -underline => 0) -> pack(-side => 'right');
@@ -198,9 +216,9 @@ for ( my $j=0;$j<$number_of_labels;$j++) {
   for (my $i=0;$i<4;$i++ ) {
 #!!!!!!!!!!!!! Entry Text
     $textfield_entry[$j][$i] = $textfield_frame[$j] -> Text(-background => "#ffffff",
-							     -font => r16,
-							     -height => 4,
-							     -width => 46)
+							    -font => r16,
+							    -height => 4,
+							    -width => 46)
       -> pack( -fill => "both", -ipadx => 10, -ipady => 10 );
 
     if (defined $dummy[$j][$i]) {
@@ -239,6 +257,22 @@ $textfield_frame[1] -> Button(-text => "clear",
 MainLoop;
 
 #######################
+# Very incomplete :-(
+#
+sub create_canvas {
+# First we need a new Window to show the objects
+
+ my $cmw = MainWindow -> new;
+
+  $cmw -> title("Move oobjects of the labels");
+  $canvas = $cmw -> Scrolled('Canvas')->pack(  );
+
+# The inner circle
+my $inner_crcle = $canvas->createOval(10,10,100,100, -extent => 360);
+
+
+}
+
 sub add_font_from_entry {
 
   $fontlistbox -> insert('end', $fontentry -> get() );
@@ -268,7 +302,7 @@ sub clear_textfields {
 #######################
 sub configure_fonts {
   # It is not possible to use the size from the perl-tk fonts, because the perl-tk
-  # uses the nearest matching font on the system but we want exact postscript fonts
+  # uses the nearest matching font on the system but we want exact PostScript fonts
 
   my $j = shift;
 
@@ -324,12 +358,15 @@ sub defaults {
   # Number of label on a sheet (very incomplete use of this variable)
   $number_of_labels = 2;
 
-  # A variable to define the backround wich is treated as an additional circle or textfield
+  # A variable to define the backround which is treated as an additional circle or textfield
   $number_of_background = 99;
 
   # which previewer
   $viewer = "/usr/X11R6/bin/gv";
 
+  # which converter
+  $converter = "/usr/bin/convert";
+
   # defaults for the circles in mm
   $radius_outer = 60;
   $radius_inner = 21;
@@ -578,7 +615,6 @@ sub evaluate_configuration {
 
   if (defined($symtab{$var})) {
      my($eval) = qq|\$$var = "$value"|;
-#    print  "Process \$var $var; \$value $value ; eval: $eval\n";
      eval $eval;
    }
 }
@@ -596,9 +632,13 @@ sub preview {
 ########################
 sub print_circles {
 
+    my $dummy;
+    my $name;
+    my $ext;
+    my $path;
   my $label_no;
 
-  print PS_FILE "%%Page: 1 1\n\n"; # begin of page
+  print PS_FILE "%%Page: 1 1\n\n"; # beginning of page
 
   print PS_FILE "gsave\n";
 
@@ -617,7 +657,20 @@ sub print_circles {
     print PS_FILE "newpath\n";
 
     # Then we print the graphics
-    if ($eps_file_label[$label_no]) {print_graphics($eps_filename[$label_no],$label_no);}
+    if ($eps_file_label[$label_no]) {
+	# check wether it is an eps-file
+	$dummy =`file $eps_filename[$label_no]`;
+	if ($dummy =~ /EPS/) {
+	    print_graphics($eps_filename[$label_no],$label_no);
+	}
+	else {
+	    $dummy = $eps_filename[$label_no];
+	    ($name,$dir,$ext) = fileparse($dummy);
+	    $dummy = "/tmp/".$name.".eps";
+	    system "$converter $eps_filename[$label_no] $dummy";
+	    print_graphics($dummy,$label_no);
+	}
+    }
 
     # After that we need the small inner circle in white
     print PS_FILE "1 1 1 setrgbcolor\n";
@@ -696,7 +749,7 @@ sub print_graphics {
   open (EPS_FILE,"<$file");
   while ( <EPS_FILE>) {
 # We should scan the graphics file for "Showpage"
-    print PS_FILE $_ ;
+    unless ($_ =~ /showpage/) {print PS_FILE $_ ;}
   }
   close (EPS_FILE);
 
@@ -819,6 +872,7 @@ sub print_prolog {
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % A function to print the textfields properly
+% As you can see, this code is taken from somewhere
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 % Zeilenumbruch
@@ -940,11 +994,11 @@ CirctextDict begin
   /FindHalfangle                 % 'FindHalfangle' calculates the length of a string
   { stringwidth pop 2 div        % on the circle
     2 XRadius mul Pi mul div     % The calculation divides the length of the text
-    360 mul                      % with the length of the circle. The half of the result
+    360 mul                      % by the circumference of the circle. Half of the result
   } def                          % will be put on the stack
 
   /Outsideplacechar              % put one character on outside of the arc
-  { /Char exch def               % and rotates the width of the character
+  { /Char exch def               % and rotate the width of the character
     /Halfangle
     Char FindHalfangle def
     gsave
@@ -1016,7 +1070,7 @@ sub print_textfields {
 	# Ok, here we are, we have to correct the current point by the half of
 	# the fontsize
  	print PS_FILE "\n0 -", $scalefont[$i+4]/2 , " rmoveto\n";
-	# Only the textfields above and below the hole in  the middle should be centered
+	# Only the textfields above and below the hole in the middle should be centered
 	if ($i == 0 || $i ==2) {
 	  print PS_FILE "center ";
 	}
@@ -1038,12 +1092,17 @@ sub print_ticks {
   my $y_origin = 0;
 
   print PS_FILE $begin_from_left[0] / 2 ," mm ", 2*$radius_outer + $delta / 2 + $begin_from_bottom , " mm moveto\n";
-#  print PS_FILE $x_origin ," mm ", $y_origin ," mm moveto\n";
   print PS_FILE "gsave\n";
   print PS_FILE "/Helvetica findfont 12 scalefont setfont\n";
   print PS_FILE "90 rotate\n";
-  print PS_FILE "(The tickmarks are in mm\\n";
-  print PS_FILE "All complaints to matthias\@tyche.de please) center show\n newpath\n";
+  print PS_FILE "(The tickmarks are in mm) center show\n";
+  print PS_FILE "grestore\n";
+  print PS_FILE "gsave\n";
+  print PS_FILE "newpath\n";
+  print PS_FILE "/Helvetica findfont 12 scalefont setfont\n";
+  print PS_FILE $begin_from_left[0] / 2 + 10," mm ", 2*$radius_outer + $delta / 2 + $begin_from_bottom , " mm moveto\n";
+  print PS_FILE "90 rotate\n";
+  print PS_FILE "(All complaints to matthias\@tyche.de please) center show\n";
   print PS_FILE "grestore\n";
 
   print PS_FILE "gsave\n";
@@ -1141,7 +1200,7 @@ sub get_bounding_box() {
     close ($eps_file);
   }
   else {
-    warning_window("There seems no bounding box in $eps_file\nHope the best\n");
+    warning_window("There seems no bounding box in $eps_file\nHope for the best\n");
   }
 }
 
@@ -1157,12 +1216,17 @@ sub get_eps_file {
 
 # set the filename if defined as default !!!!!!!!!!!
   $FSref = $mw -> FileSelect(-directory => $graphics_directory,
-			     -defaultextension => "eps",
+#			     -defaultextension => "eps",
 			     -initialfile => $dummy);
 
   $graphics_file = $FSref -> Show;
 
   if ( defined $graphics_file) {
+# Remember the path 
+# First get it from the string
+  $graphics_file =~ /(.*\/)(.*)/;
+# and store it  
+  $graphics_directory = $1;
     for ( $i=0;$i < $number_of_labels;$i++) {
       if ($eps_file_label[$i]) {
 	$eps_filename[$i] = $graphics_file;
@@ -1196,7 +1260,7 @@ sub get_fontattributes {
 						      -anchor => 'e',
 						      -expand => '1');
 
-# This frame ois divided in two frames
+# This frame is divided into two frames
 
   $fwc = $fw -> Frame() -> pack(-side => 'left',
 				-fill => 'both');
@@ -1231,17 +1295,17 @@ sub get_fontattributes {
 		     -variable => \$which_circle,
 		     -command => sub{set_fontattributes()}) ->pack(-expand => 'x',
 								    -anchor => 'w');
-  $fwt -> Radiobutton(-text => "Left textfield",
+  $fwt -> Radiobutton(-text => "Right textfield",
 		     -value => 5,
 		     -variable => \$which_circle,
 		     -command => sub{set_fontattributes()}) ->pack(-expand => 'x',
 								   -anchor => 'w',);
-  $fwt -> Radiobutton(-text => "right textfield",
+  $fwt -> Radiobutton(-text => "Bottom textfield",
 		     -value => 6,
 		     -variable => \$which_circle,
 		     -command => sub{set_fontattributes()}) ->pack(-expand => 'x',
 								    -anchor => 'w');
-  $fwt -> Radiobutton(-text => "Bottom textfield",
+  $fwt -> Radiobutton(-text => "Left textfield",
 		     -value => 7,
 		     -variable => \$which_circle,
 		     -command => sub{set_fontattributes()}) ->pack(-expand => 'x',
@@ -1262,14 +1326,14 @@ sub get_fontattributes {
 
 
 ############################################################
-# Now stop the frame an put the close-button in the window
+# Now stop the frame and put the close-button in the window
 ############################################################
 
   $fwb -> Button(-text => "close",
 		 -command => sub{$fmw -> destroy() if Tk::Exists($fmw)}) -> pack;
 
 ############################################################
-# An other frame for the fontlist
+# Another frame for the fontlist
 ############################################################
 
   $flb = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
@@ -1281,7 +1345,7 @@ sub get_fontattributes {
 
 ############################################################
 # we should have these fonts with ghostscript
-# feel free to write a subroutine to fill the listbox with fonts that real exist on the local machine
+# feel free to write a subroutine to fill the listbox with fonts that really exist on the local machine
 ############################################################
 
   foreach $i  ( @fontnames) {
@@ -1379,11 +1443,6 @@ sub get_graphics {
 						       -expand => 'x',
 						       -anchor => 'w',);
 
-# $file_dialog_button
-#  $eps_file_entry = $fwf -> Entry(-text => "EPS-File",
-#				  -textvariable => \$graphics_file,
-#				  -background => "#ffffff") -> pack();
-
   for ($i=0;$i < $number_of_labels; $i++) {
     $eps_file_entry[$i] = $fwf -> Entry(-text => "EPS-File $i",
 					-textvariable => \$eps_filename[$i],
@@ -1423,6 +1482,19 @@ sub get_graphics {
 		  -command => sub{$fmw -> destroy() if Tk::Exists($fmw)}) -> pack(-side => 'bottom');
 }
 
+
+########################
+sub get_converter {
+    my $dummy;
+    my $FSref;
+    
+    $FSref = $mw -> FileSelect(-directory => "/usr/bin");
+    $dummy = $FSref -> Show;
+
+    $converter = $dummy if defined $dummy;
+}
+########################
+
 ########################
 sub get_previewer {
   my $dummy;
@@ -1576,33 +1648,55 @@ sub set_distances {
 
   if (! Exists($spw)) {
     my $spw = $mw -> Toplevel();
-    $spw -> title("Change some distance to fit the sheet");
+    $spw -> title("Change some distances to fit the sheet");
     $spw -> Button(-text => "OK",
 		   -command => sub {$spw -> withdraw}) -> pack(-side => 'bottom');
-    $spw -> Label(-text => "Here you can move the circle around the page.\nUse the tickmarks\n") -> pack;
+    $spw -> Label(-text => "Here you can move the circle around the page.\nUse the tickmarks\n",
+		 -relief => 'raised',
+		 -border  =>"3") -> pack(-fill =>'both');
 
-    my $fwt = $spw -> Frame(-relief => 'sunken') -> pack(-side => 'top');
+    my $fwt = $spw -> Frame(-relief => 'raised',
+			   -border => "3") -> pack(-side => 'top',
+						  -fill => 'both');
 
     for (my $label_no = 0;$label_no < $number_of_labels; $label_no++ ) {
+      $fwt -> Entry(-textvariable => \$begin_from_left[$label_no],
+		    -text => "Labeltext"
+		   -width => '5') -> pack(-side => 'bottom');
       $fwt -> Scale(-label => "move circle $label_no horizontal",
 		    -orient => 'horizontal',
 		    -length => 250,
 		    -resolution => 0.1,
+		    -tickinterval => 75,
 		    -variable => \$begin_from_left[$label_no],
 		    -from => 0, -to => 250,) -> pack(-side => 'bottom');
     }
 
 # We want a frame for the scales
-    my $fw = $spw -> Frame(-label => "vertical scales") -> pack(-side => 'bottom');
+    my $fw = $spw -> Frame(-relief => 'raised',
+			   -border => "3",
+			   -label => "vertical scales") -> pack(-side => 'bottom',
+							       -fill => 'both');
+
+    $fw->Entry(-textvariable => \$begin_from_bottom,
+	       -text => "Bottom",
+	      -width => '5') -> pack(-side => 'left');
 
     $fw -> Scale(-label => "start from bottom",
 		 -variable => \$begin_from_bottom,
 		 -resolution => 0.1,
 		 -from => 200, -to => -50 ) -> pack(-side => 'left');
+
+
     $fw -> Scale(-label => "delta between the circles",
 		 -variable => \$delta,
 		 -resolution => 0.1,
 		 -from => 100, -to => 0 ) -> pack(-side => 'right');
+
+    $fw->Entry(-textvariable => \$delta,
+	       -width => '5',
+	       -text => "Delta") -> pack(-side => 'right');
+
   }
   else {
     $spw -> deiconify();
@@ -1655,6 +1749,17 @@ sub write_configuration {
   close (CONF_FILE);
 }
 
+########################
+sub write_form_data {
+
+  my $home = $ENV{'HOME'};
+  my $date = localtime;
+
+  open (CONF_FILE , ">> $home/.cd-circleprint.conf") or print  ("I'm sorry, I wasn't able to write the configuration.");
+
+  close (CONF_FILE);
+}
+
 ########################
 sub dummy {
 
@@ -1696,53 +1801,56 @@ You can specify an alternate configurationfile in the commandline
 =head1 DESCRIPTION
 
 cd-circleprint prints two round shaped labels for CDs. It uses up to four lines of text for each
-label. This text is printed in arcs so the text can be much longer than in straight lines.
+label. This text is printed in arcs so the text can be much longer than if it were in straight lines.
+Furthermore there are four textfields for each label.
 
-The output is in postscript, this should work on most systems.
+The output is in PostScript, this should work on most systems.
 
-You can choose the font the size and the color of the text for each line separately. And you
+You can choose the font, size, and color of the text for each field separately, and you
 can choose the background color of the label itself. If you wish more complicated backgrounds
 or color effects you can save a label and use in any graphic program which can read
-postscript. (I suggest "The Gimp")
+PostScript. (I suggest "The Gimp")
 
 =head1 LAYOUT
 
-All layout is done in postscript, this should work on most systems.
+All layout is done in PostScript, this should work on most systems.
 
-The resulting layout are two cd-labels on one sheet of paper. In each label there are two
+The result of the layout are two cd-labels on one sheet of paper. In each label there are two
 convex and two concave lines of text. If you have a very long line of text, you can use only
-one line and make it longer, there are no limits for the length of text.
-But after 360 degrees the text is overlapping with the beginning.
+one line and make it longer.  There are no limits to the length of text.
+However, after 360 degrees the ends of the text will overlap.
 
-Furthermore there are four textfields in each label which are simply straight forward.
+There are also four textfields in each label with normal horizontal
+lines of text.  These textfields are specified in clockwise order:
+above, right, below, and left of the central hole.
 
-The colors are done in the RGB-model of Postscript, you can choose them for each line and 
+The colors are done in the RGB-model of PostScript, you can choose them for each line and 
 the background of the label independently.
 
-You can move the circles around the sheet to fit them on the most types of label-sheets.
+You can move the circles around the sheet to fit them on most types of label-sheets.
 
 You can also squeeze or stretch the whole thing to correct small errors of your printer.
 
 =head1 USING
 
-After starting the program you can see sixteen text-entry fields divided in two blocks. 
-The upper block is for the textfields the lower block for the arcs.
+After starting the program you can see sixteen text-entry fields divided into two blocks. 
+The upper block is for the textfields, the lower block for the arcs.
 
 Each label has four textfields and four arcs.
 The first two lines are for the upper arcs on the label and the last two for the lower arcs.
 
-Write your text in the fields (support for a cddb is on the TODO list)
+Write your text in the fields (support for cddb is on the TODO list)
 and press the button "label" or press Control-l in the menu on top of the window. This button creates a file
-/tmp/cd-circleprint.ps which can be previewed with an postscript viewer. There is a button to start
+/tmp/cd-circleprint.ps which can be previewed with an PostScript viewer. There is a button to start
 your favorite previewer or you can press Control-p.
 
 Font attributes and colors can changed interactively with the menu.
 
 =head2 Shortcuts
-Control-l: write the label in /tmp
-Control-p: preview the label
-Control-q: quit
-Control-s: save the configuration
+	Control-l: write the label in /tmp
+	Control-p: preview the label
+	Control-q: quit
+	Control-s: save the configuration
 
 =item B<Label-Options>
 
@@ -1750,7 +1858,7 @@ Control-s: save the configuration
 Here you can change fontfamily, fontsize, font-color and the backgroundcolor interactively, all
 changes should take effect immediately in the text entry fields.
 
-Be carefully, perl-tk uses the nearest match for the font maybe the labels look different on your printer.
+Be carefully, perl-tk uses the nearest match for the font. Maybe the labels will look different on your printer.
 
 =item I<Radii>
 There are many types of CD-labels, so here you are able to correct the size of the inner circle.
@@ -1764,21 +1872,19 @@ the whole page to get perfect circles with your printer.
 =item I<tickmarks>
 There are many cd-label sheets on the market, so you can move the circles around the page. To make this
 easier, you can print some tickmarks on the page to measure the distances.
-If you have the correct values, you don't need this tickmarks any more can save some ink.
+If you have the correct values, you don't need these tickmarks any more and can save some ink.
 
 =item I<distances>
 You can move the circles around the page with this menu.
 
 =item I<graphics>
-You can put some graphics on the label. This graphics has currently to be in eps-format.
-Be careful, I had problems with some eps-graphics created by The Gimp, which had a "showpage" in it.
-After deleting this line everything worked fine. I will look if this is a feature or a bug.
+You can put an image on the label. This image currently has to be in eps-format.
 
 I thought about automatic scaling, but the program would
-have to know the real size of the visible part of the graphic but it knows only the bounding box.
+have to know the real size of the visible part of the graphic instead of just the bounding box.
 
 The scaling and shifting should be self-explanatory. The programs get the bounding box of the eps and 
-puts the graphics in the middle of the label.
+puts the image in the middle of the label.
 
 =head1 CONFIGURATION
 
@@ -1791,18 +1897,18 @@ Not all options are stored now, but I'm working on it.
 There is a global configuration file /etc/cd-circleprint.conf and a local configuration file 
 $HOME/.cd-circleprint.conf
 
-You can specify an other configuration file on the commandline, than the default configuration files are ignored.
+You can specify another configuration file on the commandline, then the default configuration files are ignored.
 For the format of the configuration files take look at them.
-comments start with a # and blank lines are ignored.
+Comments start with a # and blank lines are ignored.
 
 The program sets its internal default values, looks for the global configuration and then
 for the local configuration file.
 
 =head1 BUGS
 
-There are sure a many bugs and much more ugly code but it works.
+There are surely many bugs and much more ugly code but it works.
 Please fix them and send me a patch.
-Any native english speaker who wants to correct the bad English of this man-page would be welcome too.
+Any native English speaker who wants to correct the bad English of this man-page would be welcome too.
 Thanks
 
 =head1 AUTHORS
@@ -1810,7 +1916,11 @@ Thanks
 cd-circleprint is written and maintained by Matthias Kabel <matthias.kabel@tyche.de>
 
 You can find the latest version on http://cd-circle-print.sourceforge.net
-The program is licensed under the GNU Public License version 2 or higher
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
 Thanks to sourceforge.net for hosting free software
 
